# -*- coding: utf-8 -*-
##############################################################################
#    
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.     
#
##############################################################################

import collections
import base64
import time
import gtk
import rpc


def make_pixbuf(data):
    ''' Generate a pixbuf (gnome image) from a image file data
     Warning this processing is slow and should be cached'''
    if not data:
        data = file(common.terp_path_pixmaps("noimage.png"), 'rb').read()            
    else:
        data = base64.decodestring(data)
    pixbuf = None
    for type in ('jpeg', 'gif', 'png', 'bmp'):
        loader = gtk.gdk.PixbufLoader(type)
        try:
            loader.write(data, len(data))
        except:
            continue
        pixbuf = loader.get_pixbuf()
        if pixbuf:
            break
    if not pixbuf:
        loader = gtk.gdk.PixbufLoader('png')
        loader.write(NOIMAGE, len(NOIMAGE))
        pixbuf = loader.get_pixbuf()

    loader.close()
    return pixbuf

def thumbnail_pixbuf(pixbuf, to_width, to_height):
    ''' Make a thumbnail of the image according to dimensions provided  
     Warning this processing is slow and should be cached'''
    img_height = pixbuf.get_height()
    if img_height > to_height:
        height = to_height
    else:
        height = img_height

    img_width = pixbuf.get_width()
    if img_width > to_width:
        width = to_width
    else:
        width = img_width

    if (img_width / width) < (img_height / height):
        width = float(img_width) / float(img_height) * float(height)
    else:
        height = float(img_height) / float(img_width) * float(width)

    return pixbuf.scale_simple(int(width), int(height), gtk.gdk.INTERP_BILINEAR)
       
class CacheDict(collections.MutableMapping):
    ''' Act like a dict but with timeout
        @param timeout: timeout in seconds 
    '''
    def __init__(self, timeout=30):
        self.cache = {}
        self.timeout = int(timeout)
        self.last_check = time.time()
        
    def __setitem__(self, key, value):
        if not self.timeout:
            return
        self.check_timeout()
        self.cache[key] = (value, time.time())
      
    def __getitem__(self, key):
        if key not in self.cache:
            raise KeyError        
        return self.cache[key][0]
    
    def __delitem__(self, key):
        if key not in self.cache:
            raise KeyError
        del self.cache[key]
        
    def keys(self):
        return self.cache.keys()
    
    def __iter__(self):
        for key in self.cache:
            yield key
    
    def __len__(self):
        return len(self.cache)
        
    def check_timeout(self):
        ''' Remove expired cache '''
        now = time.time()
        if  now < self.last_check + self.timeout / 10:
            return
        ct = now - self.timeout # Critical time
        key2del = [k for k in self.cache if self.cache[k][1] < ct]        
        for key in key2del:
            self.cache.pop(key)
            
CACHE_TIME_OUT = 120 # In second
pixbuf_cache = CacheDict(CACHE_TIME_OUT)


def get_pixbuf(resource, field, id):
    ''' Retrieve the data and convert to pixbuf if possible
    @param resource: The resource the image come from. (Ie: res.images)
    @param field: The column of the resource. (Ie: img)
    @param id: Id of the record row in database.
    '''
    cache_id = "%s[%s][%d]" % (resource, field, id)
    if cache_id in pixbuf_cache:
        return pixbuf_cache[cache_id]
    data = None
    if id != False:
        res = rpc.session.rpc_exec_auth('/object', 'execute', resource, 'read', 
                                        id, [field])
        if res:
            data = res[field]
    pixbuf = make_pixbuf(data)
    pixbuf_cache[cache_id] = pixbuf
    return pixbuf

def get_thumbnail(resource, field, id, to_width, to_height):
    ''' Resize Image to fit width, height cache result to be used later.
    @param resource: The resource the image come from. (Ie: res.images)
    @param field: The column of the resource. (Ie: img)
    @param id: Id of the record row in database.
    @param to_width: Maximum width in pixel.
    @param to_height: Maximum height in pixel.
    '''
    cache_id = "%s[%s][%d]-%dx%d" % (resource, field, id, to_width, to_height)
    if cache_id in pixbuf_cache:
        return pixbuf_cache[cache_id]
    pixbuf = get_pixbuf(resource, field, id)
    thumbnail = thumbnail_pixbuf(pixbuf, to_width, to_height)
    pixbuf_cache[cache_id] = thumbnail
    return thumbnail